# Copyright (c) 2003 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
Aspect Implementation of Logger
"""

__revision__ = '$Id: logger.py,v 1.9 2003/06/16 15:23:39 adim Exp $'


from logilab.aspects.core import AbstractAspect
from logilab.aspects.prototypes import reassign_function_arguments


class LoggerAspect(AbstractAspect):
    """Logger Aspect Class.
    The Logger will log all informations related to method calls
    """
    
    def __init__(self, cls, method_name, log_device):
        """
        cls : the class or class instance to aspect
        method_name : the name of the method to wrap
        log_device : the stream to write log messages in
        """
        AbstractAspect.__init__(self, cls, method_name)
        self.log_device = log_device
        
        
    def before(self, wobj, *args, **kwargs):
        """Before method
        """
        self.log_device.write("Calling %s \n"%self.method_name)
        self.log_device.write("\twith values : \n")
        call_dict = reassign_function_arguments(self.method,args,kwargs)
        for var, val in call_dict.items():
            self.log_device.write('\t\t "%s" = %r\n'%(repr(var),val))
        

    def after(self, wobj, ret_v, exec_excpt, *args, **kwargs):
        """After method.
        Logs the return value
        """
        if exec_excpt is not None:
            self.log_device.write("End of %s. Exception was raised : %s" %
                                  (self.method_name, exec_excpt))
        else:
            self.log_device.write("End of %s. Return Value is : %r\n"% \
                                  (self.method_name,ret_v))


