# Copyright (c) 2003 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


"""
Aspect Implementation of Logger
"""

__revision__ = '$Id: profiler.py,v 1.3 2003/06/16 15:23:40 adim Exp $'


from logilab.aspects.core import AbstractAspect

import time

class ProfilerAspect(AbstractAspect):
    """Logger Aspect Class.
    The Logger will log all informations related to method calls
    """

    __profile_dict = {}
    
    def __init__(self, cls, method_name):
        """
        cls : the class or class instance to aspect
        method_name : the name of the method to wrap
        log_device : the stream to write log messages in
        """
        AbstractAspect.__init__(self, cls, method_name)
        if cls not in ProfilerAspect.__profile_dict:
            ProfilerAspect.__profile_dict[cls] = {}
        if method_name not in ProfilerAspect.__profile_dict[cls]:
            ProfilerAspect.__profile_dict[cls][method_name] = []
        
    
    def around(self, wobj, *args, **kwargs):
        """Around implementation for Profiler
        """
        
        start_time = time.time()
        try:
            return self._proceed(wobj,*args, **kwargs)
        finally:
            end_time = time.time()
            self._add_profile_entry(start_time, end_time-start_time)
        
            
    def _get_profile_entry(self):
        """Returns the correct entry in the profile dictionnary
        """
        return ProfilerAspect.__profile_dict[self.weaved_obj][self.method_name]

    def _add_profile_entry(self,call_time, exec_time):
        """Add an entry to the profile dictionnary
        call_time : the time when method was called
        exec_time : the execution time of the method
        """
        entry = ProfilerAspect.__profile_dict[self.weaved_obj][self.method_name]
        entry.append((call_time,exec_time))
        

    def dump_readable_profiles(stream):
        """Static Method to dump results in a readable way
        """
        stream.write("*"*15+"  Results  "+"*"*15+'\n')
        for cls in ProfilerAspect.__profile_dict:
            stream.write('In %r :\n'%cls)
            for met_name in ProfilerAspect.__profile_dict[cls]:
                entries = ProfilerAspect.__profile_dict[cls][met_name]
                stream.write('\t %s (called %d times):\n'% \
                             (met_name, len(entries)))
                for entry in entries:
                    stream.write('\t\t - Called at %s, exec_time = %s sec\n' % \
                                 (time.ctime(entry[0]),entry[1]))
                stream.write('\t\n')
                
        stream.write("*"*41+'\n')

    dump_readable_profiles = staticmethod(dump_readable_profiles)
