# Copyright (c) 2003 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
Unit tests for aspects
"""

__revision__ = '$Id: unittest_aspects.py,v 1.14 2003/06/16 15:23:40 adim Exp $'


import unittest

from logilab.aspects.weaver import weaver
from logilab.aspects.core import AbstractAspect



class EmptyAspect(AbstractAspect):
    """An empty aspect class to test if method wrapping
    is well done.
    """

    before_flag = 1
    around_flag = 2
    after_flag = 4
    proceed_flag = 8
    
    def __init__(self, cls, method_name, arg1, arg2):
        """arg1 and arg2 are here to test that the aspect can
        be be built with additional args
        """
        AbstractAspect.__init__(self, cls, method_name)
        self.arg1 = arg1
        self.arg2 = arg2
        # A flag to test if wrapping methods are called
        self.trace_flag = 0
        

    def before(self, wobj, *args, **kwargs):
        """Before method
        """
        self.trace_flag |= EmptyAspect.before_flag


    def after(self, wobj, ret_v, exec_excpt, *args, **kwargs):
        """After method
        """
        self.trace_flag |= EmptyAspect.after_flag


    def around(self, wobj, *args, **kwargs):
        """Around method
        """
        self.trace_flag |= EmptyAspect.around_flag
        self._proceed(wobj, *args, **kwargs)
        
        

class Foo:
    """A simple class to tests aspects
    """

    def silly_method(self):
        """Just for tests, does nothing
        """
        pass


    def raise_exception(self):
        """Just for tests, does nothing
        """
        raise NotImplementedError



class Aspect_ClassTest(unittest.TestCase):
    """TestCase for aspects
    """

    def setUp(self):
        """setUp method for tests
        """
        self.foo = Foo()
        weaver.weave_methods(self.foo, EmptyAspect, 10, 20)
        self.aspect = weaver.get_aspects(self.foo, 'silly_method')[0]
        

    def test_aspect_init(self):
        """Tests if aspect's init is well done with additional args
        """
        self.assert_(self.aspect.arg1 == 10)
        self.assert_(self.aspect.arg2 == 20)
        

    def test_method_wrapping(self):
        """Tests if methods are well wrapped
        """
        ok_val = EmptyAspect.before_flag | EmptyAspect.after_flag | \
                 EmptyAspect.around_flag
        self.foo.silly_method()
        self.assert_(self.aspect.trace_flag == ok_val)
        

    def test_exception_management(self):
        """Tests if exception are re-raised
        """
        self.assertRaises(NotImplementedError, self.foo.raise_exception)
        


def suite():
    """Callable object that returns the entire testsuite
    (useful for alltests.py)
    """
    return unittest.makeSuite(Aspect_ClassTest,"test")


if __name__ == '__main__':
    unittest.main()

